# frozen_string_literal: true

module BucketCake
  class Base
    def items
      klass = self.class::PROTOCLASS.call
      Decoder.new(files, klass).items.lazy
    end

    private

    def files
      Enumerator.new do |y|
        keys.each do |key| # rubocop:disable Performance/HashEachMethods
          y << bucket.object(key).get.body
        end
      end
    end

    def folder
      self.class::FOLDER
    end

    def bucket
      @bucket ||= Aws::S3::Bucket.new(ENV.fetch('CAKE_DATA_BUCKET'))
    end

    class Range < self
      include TimeHelper

      attr_reader :start_time, :end_time

      def initialize(start_time, end_time)
        assert_time(start_time)
        assert_time(end_time)
        raise 'Invalid time: end must be after start' unless end_time > start_time

        @start_time = start_time
        @end_time = end_time
      end

      private

      def keys
        hour_keys(folder, start_time, end_time)
      end
    end

    class Latest < self
      LATEST_FILE = 'latest.gz'

      def dataset_unique_id
        bucket.object(keys.first).etag
      end

      private

      def keys
        ["#{folder}/#{LATEST_FILE}"]
      end
    end

    class Realtime < self
      include TimeHelper

      DEFAULT_LOOKBACK = 24.hours
      CURSOR_REGEX = %r{\A\w+/\d{4}/\d{2}/\d{2}/\d{4}\.gz\z}

      attr_reader :cursor

      def initialize(cursor)
        raise 'Invalid cursor format!' unless cursor.nil? || cursor =~ CURSOR_REGEX
        @cursor = cursor || default_cursor
      end

      def files
        @keys = load_keys
        @cursor = keys.last unless keys.empty?
        super
      end

      private

      attr_reader :keys

      def default_cursor
        minute_cursor(folder, Time.now - DEFAULT_LOOKBACK)
      end

      def load_keys
        # return all objects after the cursor timestamp
        bucket.objects(marker: cursor, prefix: "#{folder}/").map(&:key)
      end
    end
  end
end
